#include "Commands_Misc.h"

#include "GameForms.h"
#include "GameObjects.h"
#include "GameAPI.h"
#include "GameRTTI.h"
#include "GameBSExtraData.h"
#include "GameExtraData.h"
#include "GameProcess.h"
#include "StringVar.h"

bool Cmd_GetAgeClass_Eval(COMMAND_ARGS_EVAL)
{
	*result = 0;
	Actor* pActor = NULL;
	TESNPC * pNPC = NULL;
	TESRace * pRace = NULL;
	char * name;
	char upperName[256];

	//DEBUG_MESSAGE("\t\tGAC Arg1:%x\n", arg1);

	if (arg1)
		pActor = DYNAMIC_CAST(arg1, TESObjectREFR, Actor);
	//DEBUG_MESSAGE("\t\tGAC Actor:%x\n", pActor);
	if (!pActor)
		if (thisObj) {
			//DEBUG_MESSAGE("\t\tGAC thisObj:%x\n", thisObj);
			pActor = DYNAMIC_CAST(thisObj, TESObjectREFR, Actor);
		}
	//DEBUG_MESSAGE("\t\tGAC Actor:%x\n", pActor);
	if (pActor)
		pNPC = DYNAMIC_CAST(pActor->baseForm, TESForm, TESNPC);
	//DEBUG_MESSAGE("\t\tGAC Actor:%x NPC:%x\n", pActor, pNPC);
	if (pNPC)
		pRace = pNPC->race.race;
	//DEBUG_MESSAGE("\t\tGAC Actor:%x Race:%x\n", pActor, pRace);
	if (pRace) {
		if (pRace->raceFlags & TESRace::kFlag_Child)
				*result = 0;
		else {
			name = pRace->fullName.name.m_data;
			//DEBUG_MESSAGE("\t\tGAC Actor:%x Race:%x Name:%x\n", pActor, pRace, name);
			if (name) {
				for (int i=0; i<=strlen(name); i++)
					upperName[i] = toupper(pRace->fullName.name.m_data[i]);
				//DEBUG_MESSAGE("\t\tGAC Actor:%x Race:%x Name:%s\n", pActor, pRace, upperName);
				if (strstr(upperName, "CHILD"))
					*result = 0;
				else if (strstr(upperName, "OLDAGED"))
					*result = 3;
				else if (strstr(upperName, "OLD"))
					*result = 2;
				else
					*result = 1;
			}
			else
				*result = 1;
			//DEBUG_MESSAGE("\t\tGAC Actor:%x Race:%x Name:%s Result:%d\n", pActor, pRace, upperName, *result);
		}
	}
	else
		*result = -1;
	if (IsConsoleMode())
		Console_Print("GetAgeClass: %.0f", *result);

	//DEBUG_MESSAGE("\t\tGAC Actor:%x Result:%f\n", pActor, *result);
	return true;
}

bool Cmd_GetAgeClass_Execute(COMMAND_ARGS)
{
	void * arg1 = NULL;
	ExtractArgs(EXTRACT_ARGS, &arg1);
	return Cmd_GetAgeClass_Eval(thisObj, arg1, 0, result);
}

bool Cmd_GetRespawn_Eval(COMMAND_ARGS_EVAL)
{
	*result = 0;
	Actor* pActor = NULL;
	TESForm * pForm = NULL;
	TESActorBaseData* pActorBaseData = NULL;

	if (arg1)
		pActor = DYNAMIC_CAST(arg1, TESForm, Actor);

	if (!pActor && thisObj)
		pActor = DYNAMIC_CAST(thisObj, TESObjectREFR, Actor);

	if (pActor)
		pForm = pActor->baseForm;

	if (pForm)
		pActorBaseData =  DYNAMIC_CAST(pForm, TESForm, TESActorBaseData);

	if (pActorBaseData)
		*result = pActorBaseData->flags & TESActorBaseData::kFlags_Respawn ? 1.0 : 0.0;

	if (IsConsoleMode())
		Console_Print("GetRespawn: %.0f", *result);

	return true;
}

bool Cmd_GetRespawn_Execute(COMMAND_ARGS)
{
	void * arg1 = NULL;
	ExtractArgs(EXTRACT_ARGS, &arg1);
	return Cmd_GetRespawn_Eval(thisObj, arg1, 0, result);
}

bool Cmd_SetRespawn_Execute(COMMAND_ARGS)
{
	*result = 0;
	double flag = 0;
	Actor* pActor = NULL;
	TESForm * pForm = NULL;
	TESActorBaseData* pActorBaseData = NULL;

	if (!ExtractArgs(EXTRACT_ARGS, &flag, &pActor))
		return true;

	if (!pActor && thisObj) 
		pActor = DYNAMIC_CAST(thisObj, TESObjectREFR, Actor);

	if (pActor)
		pForm = pActor->baseForm;

	if (pForm)
		pActorBaseData =  DYNAMIC_CAST(pForm, TESForm, TESActorBaseData);

	if (pActorBaseData) 
		flag ? pActorBaseData->flags |= TESActorBaseData::kFlags_Respawn : pActorBaseData->flags &= ~TESActorBaseData::kFlags_Respawn;

	return true;
}

bool Cmd_GetPermanent_Eval(COMMAND_ARGS_EVAL)
{
	*result = 0;
	TESObjectREFR* pObj = NULL;
	TESForm * pForm = NULL;

	if (arg1)
		pObj = DYNAMIC_CAST(arg1, TESForm, TESObjectREFR);

	if (!pObj && thisObj)
		pObj = thisObj;

	if (pObj)
		pForm = pObj->baseForm;

	if (pForm)
		*result = pForm->flags & TESForm::kFormFlags_QuestItem ? 1.0 : 0.0;

	if (IsConsoleMode())
		Console_Print("GetPermanent: %.0f", *result);

	return true;
}

bool Cmd_GetPermanent_Execute(COMMAND_ARGS)
{
	void * arg1 = NULL;
	ExtractArgs(EXTRACT_ARGS, &arg1);
	return Cmd_GetPermanent_Eval(thisObj, arg1, 0, result);
}

bool Cmd_SetPermanent_Execute(COMMAND_ARGS)
{
	*result = 0;
	double flag = 0;
	TESObjectREFR* pObj = NULL;
	TESForm * pForm = NULL;

	if (!ExtractArgs(EXTRACT_ARGS, &flag, &pObj))
		return true;

	if (!pObj && thisObj) 
		pObj = thisObj;

	if (pObj)
		pForm = pObj->baseForm;

	if (pForm)
		flag ? pForm->flags |= TESForm::kFormFlags_QuestItem : pForm->flags &= ~TESForm::kFormFlags_QuestItem;

	return true;
}

bool Cmd_GetRace_Execute(COMMAND_ARGS)
{
	TESNPC* npc = 0;
	UInt32* refResult = (UInt32*)result;
	*refResult = 0;

	if (!ExtractArgs(EXTRACT_ARGS, &npc))
		return true;

	if (!npc)
	{
		if (thisObj && (thisObj->baseForm->typeID == kFormType_NPC))
			npc = DYNAMIC_CAST(thisObj->baseForm, TESForm, TESNPC);
		else
			return true;
	}

	*refResult = npc->race.race->refID;
	return true;
}

bool Cmd_GetRaceName_Execute(COMMAND_ARGS)
{
	TESNPC* npc = 0;
	UInt32* refResult = (UInt32*)result;
	*refResult = 0;

	if (!ExtractArgs(EXTRACT_ARGS, &npc))
		return true;

	if (!npc)
	{
		if (thisObj && (thisObj->baseForm->typeID == kFormType_NPC))
			npc = DYNAMIC_CAST(thisObj->baseForm, TESForm, TESNPC);
		else
			return true;
	}

	AssignToStringVar(PASS_COMMAND_ARGS, npc->race.race->name.CStr());
	return true;
}
